rcs_ident = '$Header: /var/home/siebren/src/xfreeze/RCS/ltup.py,v 1.3 1995/10/20 10:55:46 siebren Exp siebren $'


# Reverse a sequence type
def revdict(d):
	ret = {}
	if type(d) in (type(()), type([])): keys = d
	else: keys = d.keys()
	for k in keys: ret[d[k]] = k
	return ret
# end revdict

abs_jump = ('JUMP_ABSOLUTE',)
rel_jump = ('FOR_LOOP', 'SETUP_LOOP', 'SETUP_EXCEPT', 'SETUP_FINALLY', 
	'JUMP_FORWARD', 'JUMP_IF_FALSE', 'JUMP_IF_TRUE')
cnd_jump = ('JUMP_IF_FALSE', 'JUMP_IF_TRUE')	# Always relative too.
unc_jump = ('JUMP_ABSOLUTE', 'JUMP_FORWARD')
unc_jumpr = ('JUMP_ABSOLUTE', 'JUMP_FORWARD', 'RETURN_VALUE')
name_instructions = (
	'STORE_NAME',  'DELETE_NAME', 'STORE_ATTR', 'DELETE_ATTR',
	'STORE_GLOBAL', 'DELETE_GLOBAL', 'LOAD_NAME', 'LOAD_GLOBAL',
	'LOAD_ATTR', 'IMPORT_NAME', 'IMPORT_FROM', 'ACCESS_MODE')


import opcode
op_i2s = opcode.opcode
op_s2i = revdict(opcode.opcode)


def dividebb(s):
	'Divide a piece of code in its basic blocks, returning a list of strings'
	if type(s) <> type(''): raise TypeError
	offset = 0
	leaders = [0, len(s)]
	while offset < len(s):
		op = ord(s[offset])
		if opcode.HAS_ARG(op):
			arg = ord(s[offset+1]) + 256 * ord(s[offset+2])
			offset = offset + 3
		else:
			offset = offset + 1
		name = op_i2s[op]

		# Analyse ordinary jumps
		if name in abs_jump:
			target = arg
		elif name in rel_jump:
			target = offset + arg
		else:
			target = None

		if target <> None:	# Target of any jump is a leader
			leaders.append(target)

		if name in cnd_jump:	# Opcode after a conditional jump is a leader
			leaders.append(offset)
	# end while

	leaders.sort()
	#print 'leaders:', leaders
	ret = []
	for x in range(len(leaders) - 1):
		frm, to = (leaders[x], leaders[x+1])
		if frm <> to:
			ret.append(s[frm:to])
	return ret
# end dividebb


def str2ltup(s):
	'Build a list of instruction tuples from a co_code string'
	offset = 0
	ret = []
	while offset < len(s):
		op = ord(s[offset])
		if not op_i2s.has_key(op):
			raise ValueError, 'illegal instruction ' + `ord(op)`
		if opcode.HAS_ARG(op):
			if len(s) - offset < 3:
				raise ValueError, 'missing argument at tail'
			ret.append(op_i2s[op],
				ord(s[offset+1]) + 256 * ord(s[offset+2]) )
			offset = offset + 3
		else:
			ret.append(op_i2s[op], None)
			offset = offset + 1
	return ret
# end str2ltup


def ltup2str(ltup):
	'Build a python bytecode string from a list of instruction tuples'
	mymaxint = (1<<15) - 1
	ret = ''
	for op, arg in ltup:
		if not op_s2i.has_key(op):
			raise ValueError, 'illegal instruction ' + `op`
		op = op_s2i[op]
		if type(op) <> type(0):
			raise TypeError, 'op is of ' + str(type(op)) + ': ' + `op`
		if opcode.HAS_ARG(op):
			if arg > mymaxint or arg < -mymaxint:
				raise ValueError, \
					'excessive argument to %s: %s' % (op_i2s[op], `arg`)
			ret = ret + '%c%c%c' % (op, arg&0xff, arg>>8)
		else:
			if arg <> None:
				raise ValueError, '%s with argument' % (opcode[op],)
			ret = ret + chr(op)
	# end for

	return ret
# end ltup2str


def ltuplen(ltup):
	'Return len(ltup2str(t)) without building the string'
	if type(ltup) <> type([]): raise TypeError
	len = 0
	for op, arg in ltup:
		if arg == None:
			len = len + 1
		else:
			len = len + 3
	return len
# end ltuplen


def offsetof(ltup, n):
	'Return byteposition of nth instruction'
	return ltuplen(ltup[:n])
