import magic
import marshal
import sys
import os

# Write a file containing frozen code for the modules in the dictionary.

header = """
#include "Python.h"

static struct _frozen _PyImport_FrozenModules[] = {
"""

trailer = """\
	{0, 0, 0} /* sentinel */
};

struct _frozen *PyImport_FrozenModules = _PyImport_FrozenModules;
"""

def mkHeader(mtime):
        return magic.magics + marshal.dumps(mtime)[1:]

def makefreeze(outfp, dict, optimize=0, strip=None, passes=2, stripdoc=1):
	done = []
	mods = dict.keys()
	mods.sort()
	for mod in mods:
		modfn = dict[mod]
		try:
			str = makecode(modfn, optimize, strip, passes,
 				       stripdoc)
		except IOError, msg:
			sys.stderr.write("%s: %s\n" % (modfn, str(msg)))
			continue
		if str:
			done.append(mod, len(str))
			writecode(outfp, mod, str)
	outfp.write(header)
	for mod, size in done:
		outfp.write('\t{"%s", M_%s, %d},\n' % (mod, mod, size))
	outfp.write(trailer)


# Return code string for a given module -- either a .py or a .pyc
# file.  Return either a string or None (if it's not Python code).
# May raise IOError.

def makecode(filename, optimize, strip=None, passes=2, stripdoc=1):
    code = None
    optimized = 0
    s=filename[:len(filename)-3]
    pyc=s+'.pyc'
    pyo=s+'.pyo'

    if filename[-3:] == '.py':
      if os.path.isfile(pyc) and not os.path.isfile(pyo):
        finfo = os.stat(filename)
        finfo2 = os.stat(pyc)
        #If .py file is older than .pyc file use the .pyc for the
        #initial code string
        if finfo[8] < finfo2[8]:
            f=open(pyc,'rb')
            try:
                f.seek(8)
                code = marshal.loads(f.read())
                print "Using pyc: ", filename
            finally:
                f.close()
      if os.path.isfile(pyo):
         finfo = os.stat(filename)
         finfo2 = os.stat(pyo)
         #If .py file is older than .pyo file use .pyo for the optimized
         #code string
         if finfo[8] < finfo2[8]:
           f=open(pyo,'rb')
           try:
               f.seek(8)
               code = marshal.loads(f.read())
               print "Using pyo: ", filename
               optimized = 1
           finally:
               f.close()
      if code == None:
        f = open(filename, 'r')
        try:
           text = f.read()
           print "Compiling: ", filename
           code = compile(text, filename, 'exec')
        finally:
           f.close()
      if optimize and optimized == 0:
          try:
              import optimize
              print "Optimizing: ", filename
              code = optimize.optimCode(code,strip,passes, stripdoc)
          except ImportError:
              sys.stderr.write('cannot find optimizer\n')
          try:
              f=open(pyo,'wb')
              header=mkHeader(os.fstat(f.fileno())[8])
              f.seek(0)
              f.write(header) 
              marshal.dump(code,f)
              f.close()
              if os.name == 'mac':    # py_compile does this too
                      import macfs
                      macfs.FSSpec(out.name).SetCreatorType('PYTH', 'TEXT')
          except IOError:
              print sys.exc_value 
              
          
      return marshal.dumps(code)
        
    if filename[-4:] == '.pyc' or filename[-4:] == '.pyo':
        f = open(filename, 'rb')
        try:
            f.seek(8)
            str = f.read()
        finally:
            f.close()
        return str
    # Can't generate code for this extension
    return None


# Write a C initializer for a module containing the frozen python code.
# The array is called M_<mod>.

def writecode(outfp, mod, str):
	outfp.write('static unsigned char M_%s[] = {' % mod)
	for i in range(0, len(str), 16):
		outfp.write('\n\t')
		for c in str[i:i+16]:
			outfp.write('%d,' % ord(c))
	outfp.write('\n};\n')


# Test for the above functions.

def test():
	import os
	import sys
	if not sys.argv[1:]:
		print 'usage: python freezepython.py file.py(c) ...'
		sys.exit(2)
	dict = {}
	for arg in sys.argv[1:]:
		base = os.path.basename(arg)
		mod, ext = os.path.splitext(base)
		dict[mod] = arg
	makefreeze(sys.stdout, dict)

if __name__ == '__main__':
	test()
