#!/usr/local/bin/python
# Optimise python code

rcs_ident = '$Header: /var/home/siebren/src/freeze/RCS/om.py,v 1.5 1995/10/15 19:04:44 siebren Exp $'

import marshal, os, sys, types

from magic import magics
import optimize

USAGE = '''Usage: %s [-rsv] [-p #passes] file...
Produce optimized .pyo files from .pyc files or python scripts.
\t-r: Rename the resulting .pyo file to .pyc files.
\t-s: Strip the linenumber administration.
\t-v: report the name of each inputfile being processed, or even more.
\t-p: Set the number of passes, default 2.
''' % (os.path.basename(sys.argv[0]),)

def mkHeader(mtime):
	return magics + marshal.dumps(mtime)[1:]
# end mkHeader


def rname(name):
	'''Interpret the name of the inputfile, open it, and read
	or create a header and a code object from it. Open a suitable
	file for our own output and return a tuple of the format:
	(<type 'string'>, <type 'code'>, <type 'file'>)
	tuple, or a string containing an error message.'''
	if name[-4:] == '.pyo':
		return 'cannot read files whoes name end in ".pyo"'
	elif name[-4:] == '.pyc':
		mode = 'rb'
		outname = name[:-1] + 'o'
	elif name[-3:] == '.py':
		mode = 'r'
		outname = name + 'o'
	else:
		mode = 'r'
		outname = name + '.pyo'

	try:
		f = open(name, mode)
	except IOError, reason:
		return 'cannot open ' + name + ' ' + `reason`

	code = None
	if mode == 'rb':
		header = f.read(8)
		if header[:4] <> magics:
			return name +  ' has bad magic'
		code = marshal.load(f)
	else:
		header = mkHeader(os.fstat(f.fileno())[8])
		code = compile(f.read(), name, 'exec')
	f.close()

	if type(code) <> types.CodeType:
		return 'no code object in ' + name + ", it's a " + type(code)

	try:
		outf = open(outname, 'wb')
		outf.write(header[:4] + '\0' * 4)	# zap mtime
	except IOError:
		return 'cannot write ' + outname

	return header, code, outf
# end rname


def main():
	import getopt
	import strop

	try:
		optR = optS = None
		optP = 2	# Default
		optV = 0
		options, args = getopt.getopt(sys.argv[1:], 'p:rsv')
		if not args: raise getopt.error, 'no names of inputfiles passed'
		for opt, arg in options:
			if opt == '-p':		optP = strop.atoi(arg)
			elif opt == '-r':	optR = 1
			elif opt == '-s':	optS = 1
			elif opt == '-v':	optV = optV + 1
			elif opt == '-?':
				raise getopt.error
			else:
				raise getopt.error, 'unanticipated option ' + opt
	except getopt.error, err:
		if type(err) <> types.StringType:
			err = ''		# Suppress error
		elif err[-1] <> '\n':
			err = err + '\n'	# Append missing newline
		sys.stderr.write(sys.argv[0] + ': ' + err + USAGE)
		sys.exit(2)

	for name in args:
		x = rname(name)
		if type(x) == type(''):
			print x
			sys.exit(1)
		header, code, out = x

		if optV: print 'optimizing', name

		code = optimize.optimCode(code, optS, optP, None, optV - 1)
		marshal.dump(code, out)
		# Rewrite header (the original is bogus
		# to detect half-way written files)
		out.seek(0)
		out.write(header)
		out.close()
		if os.name == 'mac':	# py_compile does this too
			import macfs
			macfs.FSSpec(out.name).SetCreatorType('PYTH', 'TEXT')
		if optR:
			os.rename(out.name, out.name[:-1] + 'c')

	sys.exit(0)
# end main

if __name__ == '__main__': main()
