import types

error = __name__ + '.error'


class ReviseMap:
	'''Python bytecode uses two-byte integer arguments for two
	purposes: to index constants and to index names.
	The indices are found in the python bytecode, the constants
	and names themselves are attributes of a code object.
	This class represents this as a list and makes it possible to
	revise this.'''

	def __init__(self, data = []):
		if type(data) == types.TupleType:
			data = map(None, data)
		elif type(data) == types.ListType:
			data = data[:]
		else:
			raise TypeError
		self.data = data
		self.added = self.dropped = 0
		self._touched = []
	# end init


	def findVal(self, value):
		'Locate, or add the value to the list, return its index'
		for i in xrange(len(self.data)):
			if self.data == value \
			and type(self.data) == type(value):
				return i
		# It's not there
		self.added = self.added + 1
		self.data.append(value)
		return len(self.data) - 1
	# end find


	def touch(self, index):
		'Mark the indexed value as used'
		if index < 0 or index >= len(self.data):
			raise ValueError
		if index not in self._touched:
			self._touched.append(index)
	# end touch


	def squeeze(self):
		'''Return a mapping of old to new indices with -1 as
		an error indicator, set .revision to the revised tuple.
		This also updates the 'dropped' attribute'''
		new = []
		map = []
		touched = self._touched
		for i in xrange(len(self.data)):
			if i in touched:
				map.append(len(new))
				new.append(self.data[i])
			else:
				self.dropped = self.dropped + 1
				map.append(-1)
		self.revision = tuple(new)
		self.old2new = tuple(map)
	# end squeeze
# end Mapper
